/*=============================================================================
 * 
 * Robot Framework 4
 * ColumnCounts object
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 16 september 2015: EHVL: creation
 *
 *=============================================================================
 */

/*------------------------------------------------------------------------------
 * ColumnCounts constructor
 *------------------------------------------------------------------------------
 */
function ColumnCounts() {
    this._data = null;
    this._empty = {};
    this._single = {};
}

/*------------------------------------------------------------------------------
 * ColumnCounts.add()
 *
 * adds a record to the cumulative totals in ColumnCounts
 *------------------------------------------------------------------------------
 */
ColumnCounts.prototype.add = function(record) {
    
    // Treat empty string as Not Available, just like R does
    for (var i in record) {
        if (record[i] === "") record[i] = "NA";
    }
    if (this._data === null) {
        this._data = record;
        
        var columns = Object.keys(this._data);
        for (var i=0; i<columns.length; i++) {
            var column = columns[i];
            this._empty[column] = (this._data[column] === "NA");
            this._single[column] = true;
        }
    } else {
        var columns = Object.keys(this._data);
        for (var i=0; i<columns.length; i++) {
            var column = columns[i];
            if (this._data[column] === "NA") {
                this._data[column] = record[column];
                this._empty[column] = (record[column] === "NA");
            } else {
                this._single[column] = this._single[column] &&
                    (record[column] === "NA" ||
                     this._data === "NA" ||
                     record[column] === this._data[column]);
            }
        }
    }
}

/*------------------------------------------------------------------------------
 * report()
 *
 * Generate report from countVector 
 * (either _empty or _single property of ColumnCounts)
 *------------------------------------------------------------------------------
 */
function report(countVector) {
    
    var columns = Object.keys(countVector);
    var count = 0;
    
    for (var i=0; i<columns.length; i++) {
        var column = columns[i];
        
        count += countVector[column];
    }
    
    return count;
}

/*------------------------------------------------------------------------------
 * ColumnCounts.countEmpty()
 *
 * Generate count of empty columns
 *------------------------------------------------------------------------------
 */
ColumnCounts.prototype.countEmpty = function() {
    
    return report(this._empty);
}

/*------------------------------------------------------------------------------
 * ColumnCounts.countSingle()
 *
 * Generate count of columns with exactly one non-NA value
 * Note: this._single counts all columns with at most one non-NA value,
 * so we must subtract the number of empty columns to get the desired result
 *------------------------------------------------------------------------------
 */
ColumnCounts.prototype.countSingle = function() {
    
    return report(this._single) - report(this._empty);
}

module.exports = ColumnCounts;
