/*=============================================================================
 * 
 * Robot Framework 4
 * Command line parsing module
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 19 january 2015: EHVL: creation
 *
 *=============================================================================
 */

/*------------------------------------------------------------------------------
 * getValues()
 *
 * Parses command line arguments. Command line arguments must be of the form
 * --<name>=<value>
 *
 * The number of occurrences of a given name/value pair is checked as well, 
 * based on the min and max arguments. Typical values of min and max are:
 * min = 0, max = 1: Optional command line argument
 * min = 1, max = 1: Required command line argument
 * 
 * If max is omitted, max = infinity is assumed; if min is omitted, min = 0.
 *
 * If the argument does not occur, null is returned.
 * If the argument occurs exactly once, its value is returned as a simple string
 * If the argument occurs more than once, its values are returned as an array.
 *------------------------------------------------------------------------------
 */
var getValues = function(
    name,   // name of the command line argument sought
    min,    // (optional) minimum number of times this argument must occur
    max     // (optional) maximum number of times this argument may occur
) {
    
    var query = "^--" + name + "=";
    var re = new RegExp(query);
    var result = [];
    
    for (var i=2; i<process.argv.length; i++) {
        if (process.argv[i].search(re) != -1) {
            result.push(process.argv[i].substring(query.length - 1));
        }
    }
    if (typeof(min) != "undefined" && result.length < min) {
        throw new Error("Not enough arguments with name \"" + name + "\"");
    }
    if (typeof(max) != "undefined" && result.length > max) {
        throw new Error("Too many arguments with name \"" + name + "\"");
    }
    
    if (result.length == 0) {
        return null;
    } else if (result.length == 1) {
        return result[0];
    } else {
        return result;
    }
}

module.exports = {
    get: getValues
};
