/*=============================================================================
 * 
 * Robot Framework 4
 * Main file
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 19 january  2015: EHVL: creation
 * 24 february 2015: EHVL: move extraction rules from database to config file
 * 27 february 2015: EHVL: moved remaining config rules from db to config file
 *                         and removed sqlite3 dependency.
 * 02 march    2015: EHVL: moved object definitions to separate files
 * 20 april    2015: EHVL: added framework version to log files.
 *
 *=============================================================================
 */

var moment = require("moment");
var path = require("path");

var commandLine = require("./commandline");
var user = require("./logger");
var tech = require("./levellogger");
var Robot = require("./robot");

// Code execution starts here
start();

/*------------------------------------------------------------------------------
 * start() - starts the robot. 
 *------------------------------------------------------------------------------
 */
function start() {
    
    var logName;
    
    // Parse command line arguments
    var dataDir = commandLine.get("data", 1, 1);
    var codeFile = commandLine.get("code", 1, 1);
    var logDir = commandLine.get("log", 0, 1);
    var logLevel = commandLine.get("loglevel", 0, 1);
    var browserLogLevel = commandLine.get("browser-loglevel", 0, 1);
    
    // Open file with site-specific code.
    var specific = require(path.resolve(codeFile));

    // Initialize log files
    if (logDir === null) {
        logDir = dataDir;
    }
    
    logName =  moment().format("YYYYMMDD") + specific.variables.robotName; 

    tech.open(path.join(logDir, logName + "tech.log"), logLevel);
    tech.message("INFO", "Run started. Arguments: " + process.argv.slice(2).join(" "));
    tech.message("INFO", "This is the CBS Robot Framework version 4");
    tech.message("INFO", "Current directory: " + process.cwd());
    tech.message("DEBUG", "Log directory: " + logDir);
    tech.message("DEBUG", "Log level: " + tech.getLogLevelName());
    tech.message("DEBUG", "Data directory: " + dataDir);
    tech.message("DEBUG", "Code file: " + codeFile);
    
    user.open(path.join(dataDir, logName + "user.log"));
    tech.message("DEBUG", "User log initialized");
    user.message("Run started. Arguments: " + process.argv.slice(2).join(" "));
    user.message("This is the CBS Robot Framework version 4");
    
    // Start robot proper
    var browserLogName = logName + "phantom.log";
    var robot = new Robot(specific, dataDir, logDir, browserLogName, browserLogLevel);
    robot.processItems();
}
