/*=============================================================================
 * 
 * Robot Framework 4
 * LinkedList object
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 02 march    2015: EHVL: split off from framework.js
 *
 *=============================================================================
 */

/*------------------------------------------------------------------------------
 * LinkedList constructor
 *------------------------------------------------------------------------------
 */
function LinkedList() {
    this._next = null;
    this._last = this;
}

/*------------------------------------------------------------------------------
 * LinkedList.insertAfter()
 *
 * Inserts new list element after the element at position and stores data in it.
 * If position is omitted, append the new element to the end of the list.
 *------------------------------------------------------------------------------
 */
LinkedList.prototype.insertAfter = function(
    data,      // Data to be added to the list
    position   // (optional) position after which data is to be added
) {
    
    // use last position as default if position is omitted
    if (typeof(position) === "undefined") {
        position = this._last;
    }
    
    // Initialize new list element
    newElement = { _data: data, _next: position._next, _prev: position }; 
    if (newElement._prev == this) {
        newElement._prev = null;
    }
    
    // Add new element to the list
    position._next = newElement;
    if (newElement._next) {
        newElement._next._prev = newElement;
    } 
    
    // Update reference to last element, if necessary
    if (position === this._last) {
        this._last = newElement;
    }
    
    // Enable call chaining
    return this;
}

/*------------------------------------------------------------------------------
 * LinkedList.print()
 *
 * Prints the data stored in each list element in order
 *------------------------------------------------------------------------------
 */
LinkedList.prototype.print = function() {
    
    for (var element = this._next; element; element = element._next) {
        console.log(element._data);
    }
}

/*------------------------------------------------------------------------------
 * LinkedList.getFirst()
 *
 * returns the position of the first element in the list
 *------------------------------------------------------------------------------
 */
LinkedList.prototype.getFirst = function() {
    
    return this._next;
}

/*------------------------------------------------------------------------------
 * LinkedList.getLast()
 *
 * returns the position of the last element in the list
 *------------------------------------------------------------------------------
 */
LinkedList.prototype.getLast = function() {
    
    return this._last;
}

/*------------------------------------------------------------------------------
 * LinkedList.getNext()
 *
 * returns the position of the element succeding the element at position
 *------------------------------------------------------------------------------
 */
LinkedList.prototype.getNext = function(
    position  // the element after this position is returned
) {
    
    return position._next;
}

/*------------------------------------------------------------------------------
 * LinkedList.getPrev()
 *
 * returns the position of the element preceding the element at position.
 *------------------------------------------------------------------------------
 */
LinkedList.prototype.getPrev = function(
    position  // The element before this position is returned
) {    
    
    return position._prev;
}    

/*------------------------------------------------------------------------------
 * LinkedList.getData()
 *
 * returns the data stored in the linked list at position
 *------------------------------------------------------------------------------
 */
LinkedList.prototype.getData = function(
    position  // The element of which the data is returned
) {
    
    return position._data;
}

module.exports = LinkedList;
