/*=============================================================================
 * 
 * Robot Framework 4
 * Logger object
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 22 january 2016: EHVL: creation of this comment block - 
 *                        previous code changes have not been tracked
 *
 *=============================================================================
 */

var fs = require("fs");
var eol = require("os").EOL;
var moment = require("moment");

// The way timestamps are recorded in the log file
const TIMESTAMP_FORMAT = "YYYY-MM-DD HH:mm:ss";

//Separator between timestamp and log message
const SEPARATOR = " ";


/*------------------------------------------------------------------------------
 * now()
 *
 * Returns the current date and time, formatted appropriately
 *------------------------------------------------------------------------------
 */
function now() {
    return moment().format(TIMESTAMP_FORMAT);
}

/*------------------------------------------------------------------------------
 * Logger constructor
 *------------------------------------------------------------------------------
 */
function Logger() {
    this._fd = null;
}

/*------------------------------------------------------------------------------
 * Logger.open()
 *
 * Opens a Logger log file
 *------------------------------------------------------------------------------
 */
Logger.prototype.open = function(filename) {
    this._fd = fs.openSync(filename, "a");
}

/*------------------------------------------------------------------------------
 * Logger.close()
 *
 * Closes a Logger log file
 *------------------------------------------------------------------------------
 */
Logger.prototype.close = function() {
    if (this._fd !== null) {
        fs.closeSync(this._fd);
        this._fd = null;
    }
}

/*------------------------------------------------------------------------------
 * Logger.message()
 *
 * Writes a log message to the log file
 *------------------------------------------------------------------------------
 */
Logger.prototype.message = function(text) {
    if (this._fd !== null) {
        fs.writeSync(this._fd, now() + SEPARATOR + text + eol);
    }
}

module.exports = Logger;
