/*=============================================================================
 * 
 * Robot Framework 4
 * Opcodes object
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 02 march    2015: EHVL: split off from framework.js
 * 31 august   2015: EHVL: opcodes now case-insensitive; improved error message
 *
 *=============================================================================
 */

var tech = require("./levellogger");

// TODO: some of these may not be present in certain contexts; it would be nice 
// to be able to have the valid contexts as part of this object.
var opcodes = {
    OPEN:      { name: "OPEN",      defaults: [ "" ] },
    CLICK:     { name: "CLICK",     defaults: [ {}, 0 ] },
    HOVER:     { name: "HOVER",     defaults: [ {}, 0 ] },
    BACK:      { name: "BACK",      defaults: [] },
    NONE:      { name: "NONE",      defaults: [] },
    SCROLL:    { name: "SCROLL",    defaults: [] },
    DUPLICATE: { name: "DUPLICATE", defaults: [] },
    REPEAT:    { name: "REPEAT",    defaults: [ 0 ] }
};

/*------------------------------------------------------------------------------
 * opcodes.parse() 
 *
 * Parses a string containing an opcode. Logs a FATAL error if invalid.
 *------------------------------------------------------------------------------
 */
opcodes.parse = function(
    newOpcodeName,   // string containing opcode to parse
    errorContextMsg  // string to add to an error message when the opcode cannot
                     // be parsed. Can be used to generate more user-friendly
                     // error messages.
) {    
    for (var i in opcodes) {
        var opcode = opcodes[i];

        if (opcode.name == newOpcodeName.toUpperCase()) {
            return opcode;
        }
    }
    tech.message("FATAL", "Unknown action '" + newOpcodeName + "' " + errorContextMsg);
}

module.exports = opcodes;
