Browser.prototype.recover = function(nTries) {
    
    var self = this;
    
    if (typeof(nTries) === "undefined") nTries = 0;
    
    return self._driver.getSession().then(function () {
        self.restart(); 
    }, 
    function() { 
        self.start(); //driver does not exist; just start
    }).then(function() {
    
        tech.debug("Recovering browser state");
        
        // make a copy of the self._history array
        var history = self._history.slice();
        
        /* Remove the last element from the history, because we want
         * to recover to the state before the last browser action.
         * We do this with a local copy because the recovery itself
         * could fail, in which case we need self._history intact. */ 
        if (history.length > 0) {
            history.pop();
        }
        return webdriver.promise.map(history, function(historyItem) {
            return self.execute(historyItem.instruction, historyItem.wait, historyItem.check, true);
        }).then(function() {
            tech.debug("Recovery complete");
            
            /* Now that the state is succesfully recovered we can remove the
             * last element from the "master" history in self._history. */
            if (self._history.length > 0) {
                self._history.pop();
            }
        });
    }).catch(function(err) {
        if (err instanceof Error 
            && (err.message.indexOf("ECONNREFUSED") != -1 
                || err.message.indexOf("Server terminated early") != -1)) {
            tech.error("Browser crashed during recovery");
            nTries += 1;
            if (nTries < N_RECOVER_TRIES) {
                return Browser.prototype.recover.call(self, nTries);
            } else {
                tech.fatal("Unable to recover after " 
                           + N_RECOVER_TRIES + " tries. Quitting.");
            }
        } else if (err instanceof Error
                   && err.message == "Internet connection failed") {
            // Internet connection failure dealt with the same as 
            // browser crash, but counted differently.
            tech.error("Connection failure during recovery");
            return self._driver.sleep(RELOAD_DELAY_MS).then(function() {
                return Browser.prototype.recover.call(self, nTries);
            });
        } else {
            tech.fatal("Uncaught error during recovery. Quitting: " + err.message);
        }
    });
}                               
 