/*=============================================================================
 * 
 * Robot Framework 4
 * scroll selector parser
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 26 february    2015: EHVL: creation
 *
 *=============================================================================
 */

var tech = require("./levellogger");

var scroll = {};

function normalizeSpaces(s) {
    
    return s.trim().replace(/\s+/g, " ").toLowerCase();
}

scroll.parse = function (scrollSelector) {
    
    var normSelector = normalizeSpaces(scrollSelector);
    var words = normSelector.split(" ");
    var i = 0;
    var negation = false;
    var comp;
    var pos;
    var unit;
    
    if (words.length == 1 && !words[0]) {
        tech.fatal("empty scroll selector");
    }
    if (words[i] == "not") {
        i += 1;
        negation = true;
    }
    if (words.length <= i) {
        tech.fatal("parse error: incomplete string");     
    }
    switch(words[i]) {
        case "at":
            comp = "=";
            break;
        case "above":
            comp = "<";
            break;
        case "below":
            comp = ">";
            break;
        default:
            tech.fatal("unknown keyword '" + words[i] + "' in scroll selector. Expecting 'not', 'at', 'above' or 'below'");
    }
    i += 1;
    if (words.length <= i) {
        tech.fatal("parse error: incomplete string");       
    }
    switch(words[i]) {
        case "top":
            pos = 0;
            unit = "percent";
            break;
        case "bottom":
            pos = 100;
            unit = "percent";
            break;
        default:
            var posReg = /^(\d+)(.*)$/;
            var result = posReg.exec(words[i]);
            if (!result || result.length < 3) {
                tech.fatal("unknown position indicator '" + words[i] + "' in scroll selector.");
            }
            pos = Number(result[1]);
            switch(result[2]) {
                case "%":
                case "":
                    unit = "percent";
                    break;
                case "px":
                    unit = "pixels";
                    break;
                default:
                    tech.fatal("unknown unit '" + result[2] + "' in scroll selector.");
            }
    }
    i += 1;
    if (words.length > i) {
        tech.fatal("parse error: string too long");
    }
    return { position: pos, unit: unit, comp: comp, negation: negation, rule: normSelector};
}


scroll.compare = function(position, maxPosition, parsedSelector) {
    
    tech.debug("Performing scroll selection with rule: " + parsedSelector.rule);
    
    var compPosition;
    if (parsedSelector.unit === "percent") {
        compPosition = 100 * (position / maxPosition);
    } else {
        compPosition = position;
    }
    tech.debug("position = " + position);
    tech.debug("maxPosition = " + maxPosition);
    tech.debug("compPosition = " + compPosition);
    tech.debug("rule Position = " + parsedSelector.position);
    
    var compResult;
    switch(parsedSelector.comp) {
        case '=': 
            compResult = compPosition == parsedSelector.position;
            break;
        case '<':
            compResult = compPosition < parsedSelector.position;
            break;
        case '>':
            compResult = compPosition > parsedSelector.position;
            break;
        default:
            tech.fatal("Internal error - illegal comparison operator in parsed scroll selector");
    }
    if (parsedSelector.negation) {
        compResult = !compResult;
    }
    
    tech.debug("Scroll selection result: " + (compResult ? "selected" : "not selected"));
    return compResult;
}

/*
console.log(parseScrollSelector(""));
console.log(parseScrollSelector("at"));
console.log(parseScrollSelector("not at"));
console.log(parseScrollSelector("at top"));
console.log(parseScrollSelector("at bottom"));
console.log(parseScrollSelector("at 90%"));
console.log(parseScrollSelector("at 200px"));
console.log(parseScrollSelector("at 20in"));
console.log(parseScrollSelector("at yop"));
console.log(parseScrollSelector("above 90%"));
console.log(parseScrollSelector("below 90%"));
console.log(parseScrollSelector("helow 90%"));
console.log(parseScrollSelector("not at 90%"));

var ps = parseScrollSelector("above 50%");
console.log(ps);
console.log(comparePosition(96, 194, ps));
console.log(comparePosition(97, 194, ps));
        
var ps = parseScrollSelector("above 50px");
console.log(ps);
console.log(comparePosition(49, 194, ps));
console.log(comparePosition(50, 194, ps));

var ps = parseScrollSelector("below 50%");
console.log(ps);
console.log(comparePosition(97, 194, ps));
console.log(comparePosition(98, 194, ps));

var ps = parseScrollSelector("not below 50%");
console.log(ps);
console.log(comparePosition(97, 194, ps));
console.log(comparePosition(98, 194, ps));

var ps = parseScrollSelector("at 50px");
console.log(ps);
console.log(comparePosition(49, 194, ps));
console.log(comparePosition(50, 194, ps));

var ps = parseScrollSelector("at bottom");
console.log(ps);
console.log(comparePosition(193, 194, ps));
console.log(comparePosition(194, 194, ps));

var ps = parseScrollSelector("not at bottom");
console.log(ps);
console.log(comparePosition(193, 194, ps));
console.log(comparePosition(194, 194, ps));
*/

module.exports = scroll;
