/*=============================================================================
 * 
 * Robot Framework 4
 * TodoItem object
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 02 march    2015: EHVL: split off from framework.js
 *
 *=============================================================================
 */

var _ = require("underscore");

/*------------------------------------------------------------------------------
 * TodoItem constructor
 *------------------------------------------------------------------------------
 */
function TodoItem(
    navContext,  // Navigation context for todo item
    extrContext, // Extraction context for todo item
    vars,        // Value of the navigation variables
    wait,        // wait function to use after executing the instruction
    check,       // check function to perform after the wait
    opcode,      // instruction opcode
    arg1,        // (optional) First instruction argument
    arg2         // (optional) Second instruction argument
) {
    
    var defaults = opcode.defaults;
    
    /* Initialize instruction (opcode + arguments), use default arguments
     * oif necessary */  
    if (defaults.length == 0) {
        this.instruction = { opcode: opcode };
    } else {
        if (typeof(arg1) === "undefined") {
            arg1 = defaults[0];
        }
        if (defaults.length == 1) {
            this.instruction = { opcode: opcode, arg1: arg1 };
        } else {
            if (typeof(arg2) === "undefined") {
                arg2 = defaults[1];
            }
            this.instruction = { opcode: opcode, arg1: arg1, arg2: arg2 };
        }
    }
    
    /* Initialize remaining properties */
    this.navigationContext = navContext;
    this.extractionContext = extrContext;
    this.wait = wait;
    this.check = check;
        
    /* A clone() is needed here, because the values of the navigation variables
     * may be changed later */
    this.vars = _.clone(vars);
}

/*------------------------------------------------------------------------------
 * TodoItem.setVariable()
 *
 * Change one of the navigation variables
 *------------------------------------------------------------------------------
 */
TodoItem.prototype.setVariable = function(
    varName,   // Name of the navigation variable to be changed
    value      // New value of this variable
) {

    if (varName && value) {
        this.vars[varName] = value;
    }
}

TodoItem.prototype.getVariables = function() {
    
    return this.vars;
}

module.exports = TodoItem;
