/*=============================================================================
 * 
 * Robot Framework 4
 * Wait parser
 *
 * (c) 2015 Statistics Netherlands
 *
 * Revision history:
 * 02 march    2015: EHVL: split off from framework.js
 *
 *=============================================================================
 */

var tech = require("./levellogger");

/*------------------------------------------------------------------------------
 * parseWait()
 *
 * The function takes a call to a wait function as argument and parses it. If
 * parsing fails, the error is logged, with the location of the error attached
 * A valid waitString is of the form waitFunc(waitArg1, waitArg2, ...)
 *------------------------------------------------------------------------------
 */
function parseWait(
    waitString,      // The string to be parsed
    errorContextMsg  // Should contain the location of the wait string
) {

    var waitSplitter = new RegExp("^([A-Za-z][A-Za-z0-9_]+)\\((.*)\\)$");
    var waitParts;

    // Deal with empty wait string
    if (!waitString) {
        return { funcName: null, args: [] };
    }
        
    // Split wait string into function name and arguments
    waitParts = waitSplitter.exec(waitString);
    if (!waitParts) {
        tech.message("FATAL", "Error parsing wait function "
                     + waitString + errorContextMsg);
    }
    
    // Parse arguments. Here eval() is used, so this is not very safe
    try {
        var args = eval("[" + waitParts[2] + "]");
    }
    catch(err) {
        tech.message("FATAL", "Error parsing wait function arguments: " + 
                     err.message + errorContextMsg);
    }
    
    return { funcName: waitParts[1], args: args };        
}

function parseWait2(waitObject, errorContextMsg) {

    // Deal with empty or missing wait object
    if (!waitObject) {
        return { funcName: null, args: {} };
    }
    
    var waitFunction = waitObject.operation;
    if (!waitFunction) {
        tech.debug("No valid wait function specified " + errorContextMsg);
        return { funcName: null, args: {} };
    }
    
    var args = {};
    for (i in waitObject) {
        if (i !== "operation" && waitObject.hasOwnProperty(i)) {
            args[i] = waitObject[i];
        }
    }
    return { funcName: waitFunction, args: args };
}

module.exports = { 
    parse: parseWait2
};
